/*

Copyright 2004 iteration::two Ltd

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

@ignore
*/
/**
 * The ServiceLocator class is a psuedo-singleton class, used by the application
 * developer to define and retrieve the services used by the application.
 *
 * @description
 * The singleton implementation of the ServiceLocator ensures that one
 * and only one class can be responsible for locating services. However,
 * because the ServiceLocator instance is defined in MXML, which requires 
 * a no-args public constructor, the singleton implementation is non-standard,
 * including a public constructor and error handling if the constructor
 * is called more than once.
 *
 * <p>
 * In your application, you should subclass ServiceLocator and define
 * your services, using code similar to the following:
 * </p>
 *
 * <p>
 * <code>
 * <cairngorm:ServiceLocator xmlns:mx="http://www.macromedia.com/2003/mxml"
 *                           xmlns:cairngorm="org.nevis.cairngorm.business.*" >
 *
 *   <mx:RemoteObject id="customerDelegate" source="org.nevis.cairngorm.samples.login.CustomerDelegate"
 *                    result="event.call.resultHandler( event )"
 *                    fault="event.call.faultHandler( event )">
 *   </mx:RemoteObject>
 *
 * </cairngorm:ServiceLocator>
 * </code>
 * <p>Assuming the above is in a file called <code>Services.mxml</code> in a <code>demo</code>
 * namespace, you can then instantiate your services locator in your Flex application, as follows:
 * <code>
 *    <demo:Services id="demoServices" />
 * </code>
 * </p>
 * <p>Services can later be located, usually in a business delegate class,
 * as shown here:
 * <code>
 *   var service = ServiceLocator.getInstance().getService( "customerDelegate" );
 * </code>
 * </p>
 *
 * <p>
 * In this way, Cairngorm removes any knowledge of the nesting of your application
 * containers, or any reference to mx.core.Application (in Flex), from your data
 * service code.
 * </p>
 */
class org.nevis.cairngorm.business.ServiceLocator extends mx.core.UIObject
{

//-------------------------------------------------------------------------

	/**
	 * Used to fetch the single instance of ServiceLocator class
	 *
	 * @description ServiceLocator is a singleton, and getInstance() is the
	 * factory method for the singleton.  If an ServiceLocator has not been
	 * created yet, the first call to getInstance() will create it.  Subsequent
	 * calls to getInstance() will return the initial instance that was created.
	 */
	 public static function getInstance() : ServiceLocator
	{
		if ( serviceLocator == undefined )
			serviceLocator = new ServiceLocator();

		return serviceLocator;
	}

//-------------------------------------------------------------------------

	/**
	 * Public no-args constructor, to allow creation from MXML
	 *
	 * @description ServiceLocator is a singleton, but we want to allow creation from
	 * MXML, so we allow creation of the single instance here. If an attempt is made
	 * to construct more than one instance, an Error is thrown.
	 */
	public function ServiceLocator()
	{
		if ( ServiceLocator.serviceLocator != undefined )
			throw new Error( "Only one ServiceLocator instance should be instantiated" );

		ServiceLocator.serviceLocator = this;
	}

//-------------------------------------------------------------------------

	/**
	 * Returns the service defined for the id, to allow services to be looked up
	 * using the ServiceLocator by a canonical name.
	 *
	 * @description If no service exists for the service name, an Error will be thrown.
	 * @param The id of the service to be returned. This is the id defined in the
	 * RemoteObject, WebService or HTTPService definition.
	 */
	public function getService( serviceId : String ) : Object
	{
		if ( this[ serviceId ] == undefined ) {
			//throw new Error( "No service found for service name " + serviceId );
		}
		return this[ serviceId ];
	}	

//-------------------------------------------------------------------------

	private static var serviceLocator : ServiceLocator;
}