/*

Copyright 2004 iteration::two Ltd

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

@ignore
*/

import org.nevis.cairngorm.control.Event;
import org.nevis.cairngorm.commands.Command; 
import org.nevis.cairngorm.control.Event; 
import org.nevis.cairngorm.control.EventBroadcaster; 

/**
 * The SequenceCommand is provided as a "psuedo-abstract" (since ActionScript 2.0
 * has no real concept of abstract classes) base-class that can be extended when
 * you wish to chain commands together for a single user-gesture, or establish
 * some simple form of decision-based workflow.
 * @description
 * <p>
 * By extending SequenceCommand, you can specify the name of an event that
 * should be broadcast to the controller (causing another command execution
 * without a further user-gesture) when the current command has completed
 * execution.
 * </p>
 *
 * <p>
 * For a command implementing the Responder interface, you may choose to
 * sequence a subsequent command on successful completion of the command,
 * in the onResult() handler, or on failure of the command in the onFault()
 * method.
 * </p>
 *
 * <p>
 * For commands that do not implement the Responder interface, you can 
 * simply chain commands by causing the sequenced command to be invoked
 * as the last action in your command's execute() method.
 * </p>
 *
 * <p>
 * Usage:
 * </p>
 *
 * <p>
 * In the constructor of a concrete SequenceCommand implementation, you
 * should set nextEvent to the name of the event that is responsible
 * for calling your subsequent command.
 * </p>
 * 
 * <p>
 * Alternatively, you can override the implicit nextEvent getter, to 
 * programmatically decide at runtime which event should be broadcast
 * next.
 * </p>
 *
 * <p>
 * Invocation of the next command in the sequence is explicitly controlled by
 * the developer, by calling the executeNextCommand() method provided in the
 * SequenceCommand base-class.  This can be called either in the body of the execute() method
 * (for synchronous sequencing) or in the body of an onResult() or onFault()
 * handler (for asynchronous sequencing, that can also support conditional
 * workflow).
 * </p>
 *
 * @see org.nevis.cairngorm.commands.Command
 * @see org.nevis.cairngorm.control.FrontController
 * @see org.nevis.cairngorm.control.Event
 * @see org.nevis.cairngorm.control.EventBroadcaster
 */
class org.nevis.cairngorm.commands.SequenceCommand implements Command 
{ 
        
    /** 
     * Abstract implementation of the execute() method.
     *
     * @description ActionScript 2.0 does not explicity support abstract methods and abstract classes, so this concrete 
     * implementation of the interface method must be overridden by the developer.
     * 
     */ 
    public function execute( event : Event ) : Void 
    { 
	    // abstract, so this method must be provided.   Rather than convolute additional framework classes to enforce
	    // abstract classes, we instead delegate responsibility to the developer of a SequenceCommand to ensure that
	    // they provide a concrete implementation of this method.
    } 

//-------------------------------------------------------------------------

    /** 
     * The name of the event that triggers the next command in the sequence.
     *
     * @description As an example, if we have a SequenceCommand called "LoginCommand" that should invoke "GetCustomerAccountsCommand"
     * from it's onResult() handler, and GetCustomerAccountsCommand is registered to the event "getCustomerAccounts"
     * we could set nextEvent = "getCustomerAccounts" in the constructor of the LoginCommand .
     * 
     * <p>
     * Alternatively, you can override the nextEvent implicit getter, and programmatically decide at runtime what the next
     * event to broadcast should be.
     * </p>
     * 
     * <p>
     * <b>Best-Practice Consideration</b>
     * </p>
     *
     * <p>
     * A best-practice you may wish to consider, is to name all events on your front controller as static strings, eg
     * public var static GET_CUSTOMER_ACCOUNTS:String = "getCustomerAccounts"; which provides compile-time checks against
     * mistyping the event name, which often can lead to a frustrating-to-trace error.
     * </p>
     */ 
    public function set nextEvent( event : String ) 
    { 
    	_nextEvent = event; 
    } 

    public function get nextEvent() : String 
    { 
      return _nextEvent; 
    } 

//-------------------------------------------------------------------------

    /**
     * Call to execute the next command in the sequence.
     * 
     * @description Called explicitly by the developer within a concrete SequenceCommand implementation, this method causes the
     * event registered with nextEvent to be broadcast, for the next command in the sequence to be called 
     * without further user-gesture.
     */ 
    public function executeNextCommand( eventData : Object ) 
    { 
       var isSequenceCommand : Boolean = ( _nextEvent != null );
       if( isSequenceCommand ) 
       { 
          EventBroadcaster.getInstance().broadcastEvent( nextEvent, eventData ); 
       } 
    } 

//-------------------------------------------------------------------------

    private var _nextEvent:String; 
} 
