/*

Copyright 2004 iteration::two Ltd

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

@ignore
*/

import org.nevis.cairngorm.control.Event;
import mx.events.EventDispatcher;

/**
 * The EventBroadcaster class is a singleton class, used by the application
 * developer to broadcast events that correspond to user gestures and requests.
 *
 * @description
 * The singleton implementation of the EventBroadcaster ensures that one
 * and only one class can be responsible for broadcasting events that the
 * FrontController is subscribed to listen and react to.
 *
 * <p>
 * Since the EventBroadcaster implements singleton access, use of the
 * singleton is simple to distribute throughout your application.  At
 * any point in your application, should you capture a user gesture
 * (such as in a click handler, or a dragComplete handler, etc) then
 * simply use a code idiom as follows:
 * </p>
 *
 * <p>
 * <code>
 * var eventObject = new Object();
 * eventObject.username = username.text;
 * eventObject.password = username.password;
 *
 * EventBroadcaster.getInstance().broadcastEvent( "login", eventObject );
 * </code>
 * </p>
 *
 * @see org.nevis.cairngorm.control.FrontController
 * @see org.nevis.cairngorm.control.Event
 */
class org.nevis.cairngorm.control.EventBroadcaster
{
	/**
	 * Used to fetch the single instance of EventBroadcaster class
	 *
	 * @description EventBroadcaster is a singleton, and getInstance() is the
	 * factory method for the singleton.  If an EventBroadcaster has not been
	 * created yet, the first call to getInstance() will create it.  Subsequent
	 * calls to getInstance() will return the initial instance that was created.
	 */
	public static function getInstance() : EventBroadcaster
	{
		if ( eventBroadcaster == undefined )
		   eventBroadcaster = new EventBroadcaster();

		return eventBroadcaster;
	}

//------------------------------------------------------------------------------

   private function EventBroadcaster()
	{
		EventDispatcher.initialize( this );
	}

//------------------------------------------------------------------------------

   /**
    * Broadcasts an application event as a result of a user gesture, with
    * associated data.
    *
    * @param eventName The name of the event that represents the user gesture
    * that has just occured.  This eventName will also have been registered
    * with the FrontController, so that it knows which Command class to
    * execute() when this event is broadcast.  A typical event name would
    * be "getProducts" or "fetchAccountBalance".  The eventName will be
    * available to the Command class as event.type, in the event passed to
    * the command.
    *
    * @param eventData Should the event that has been broadcast have
    * relevant data associated with it, this can be placed in eventData. The
    * Command class that handles this event on behalf of the controller, will
    * have access to eventData as event.data, in the event passed to the
    * command.
    */
	public function broadcastEvent( eventName:String, eventData:Object ) : Void
	{
		var event:Event = new Event();
		event.type = eventName;
		event.data = eventData;

		dispatchEvent( event );
	}

//------------------------------------------------------------------------------

	/**
	 * This method is for internal-use only; required by EventBroadcaster in
	 * order to use the underlying EventDispatcher class in Flex.
	 */
	private var dispatchEvent:Function;

	/**
	 * Mixin method from EventDispatcher. Adds a listener to the event broadcaster
	 */
	public var addEventListener:Function;

	/**
	 * Mixin method from EventDispatcher. Removes a listener from the event broadcaster
	 */
	public var removeEventListener:Function;

	private static var eventBroadcaster;

}