/*

Copyright 2004 iteration::two Ltd

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

@ignore
*/

import org.nevis.cairngorm.control.*;
import org.nevis.cairngorm.commands.*;
import mx.events.*;

/**
 * A base class for an application specific front controller,
 * that is able to dispatch control following particular user gestures to appropriate
 * command classes
 *
 * @description
 * <p>
 * The Front Controller is the centralised request handling class in a
 * Cairngorm application.  Throughout the application architecture are
 * scattered a number of EventBroadcaster.getInstance().broadcastEvent( ... )
 * method calls, that signal to the listening controller that a user gesture
 * has occured.
 * </p>
 *
 * <p>
 * The role of the Front Controller is to first register all the different
 * events that it is capable of handling against worker classes, called
 * Command classes.  On hearing an application event, the Front Controller
 * will look up its table of registered events, find the appropriate
 * command for handling of the event, before dispatching control to the
 * Command by calling its execute() method.
 * </p>
 *
 * <p>
 * The Front Controller is a base-class that collaborates with the
 * EventBroadcaster class to register and listen for events.  In a
 * Cairngorm application, the developer should create a class that
 * extends the FrontController, and in the constructor of their
 * application specific controller, they should make numerous calls to
 * addCommand() to register all the expected events with application
 * specific Command classes.
 * </p>
 *
 * <p>
 * Consider a LoginController, that is the main controller for a Login
 * application that has 2 user gestures - Login and Logout.  The application
 * will have 2 buttons, "Login" and "Logout" and in the click handler for
 * each button, one of the following methods is executed:
 * </p>
 *
 * <p>
 * <code>
 * public function doLogin() : Void
 * {
 *    EventBroadcaster.getInstance().broadcastEvent( "login", { username.text, password.text } );
 * }
 *
 * public function doLogout() : Void
 * {
 *    EventBroadcaster.getInstance().broadcastEvent( "logout" );
 * }
 * </code>
 *
 * <p>
 * We would create LoginController as follows:
 * </p>
 *
 * <p>
 * <code>
 * class LoginController extends org.nevis.cairngorm.control.FrontController
 * {
 *    public function LoginController()
 *    {
 *       initialiseCommands();
 *    }
 *
 *    public function initialiseCommands() : Void
 *    {
 *       addCommand( "login", new LoginCommand() );
 *       addCommand( "logout", new LogoutCommand() );
 *    }
 * }
 * </code>
 * </p>
 *
 * <p>
 * In our concrete implementation of a FrontController, LoginController, we
 * register the 2 events that are expected for broadcast - login and logout -
 * using the addCommand() method of the parent FrontController class, to
 * assign a Command class to each event.
 * </p>
 *
 * <p>
 * Adding a new use-case to a Cairngorm application is as simple as
 * registering the event against a command in the application Front Controller,
 * and then creating the concrete Command class.
 * </p>
 *
 * <p>
 * The concrete implementation of the FrontController, LoginController,
 * should be created once and once only (as we only want a single controller
 * in our application architecture).  Typically, in our main application, we
 * would declare our FrontController child class as a tag, which should be placed
 * above any tags which have a dependency on the FrontController
 * </p>
 *
 * <p>
 * <code>
 * <mx:Application  xmlns:control="com.domain.project.control.LoginController"	... >
 *
 *	<control:LoginController id="controller" />
 *
 *  ...
 * 
 * </code>
 * </p>
 *
 * @see: Command
 */
class org.nevis.cairngorm.control.FrontController
{

  /**
   * @description Creates a new controller for our application; never called directly, instead
   * we would new() our base-class, that is specific to our own application.
   */
    public function FrontController()
  {
    commands = new Array();
  }

//------------------------------------------------------------------------------

    /**
     * @ignore since this can be private - remove
     * Since FrontController is a listener on the EventBroadcaster, the
     * handleEvent method is publically exposed for EventBroadcaster.  However,
     * this method should not be called directly by the application developer.
     */
    private function handleEvent( event:Event ) : Void
  {
      executeCommand( event );
  }

//------------------------------------------------------------------------------

  private function executeCommand( event : Event ) : Void
  {
    var command:Command = getCommand( event.type );
    command.execute( event );
  }

//------------------------------------------------------------------------------

  /**
   * Registers a Command class with the Front Controller, against an event name
   *
   * @description
   * When an event is broadcast by EventBroadcaster that matches commandName,
   * the Command class referred to by commandRef receives control of the
   * application, by having its execute() method invoked.
   *
   * @param commandName The name of the event that will be broadcast by the
   * EventBroadcaster, when a particular user gesture occurs, eg "login"
   *
   * @param commandRef An instance of a Command class upon which execute()
   * can be called when the Front Controller hears an event broadcast with
   * commandName.  Typically, this argument is passed as "new LoginCommand()"
   * or similar.
   */
  public function addCommand( commandName:String, commandRef:Command ) : Void
  {
    commands[ commandName ] = commandRef;
    EventBroadcaster.getInstance().addEventListener( commandName, this );
  }

//------------------------------------------------------------------------------

  private function getCommand ( commandName ) : Command
  {
    var command:Command = commands[ commandName ];
    if ( command == null )
      throw new Error( "Command not found for " + commandName );

    return command;
  }

//------------------------------------------------------------------------------

  private var commands:Array;
}
