/*

Copyright 2004 iteration::two Ltd

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

@ignore
*/
import org.nevis.cairngorm.view.ViewLocator;
import mx.utils.Delegate;

/**
 * Used to isolate command classes from the implementation details of a view
 *
 * @description
 * In order to carry out their function, Command classes will require
 * to both interrogate and update the view.  Prior to performing any
 * business logic, the Command class may require to fetch values that
 * have been set on the view; following completion of any business logic,
 * the final task often to be performed of a Command class is to update
 * the View (user interface) with any results returned, or perhaps to
 * switch the View entirely (to a different screen).
 *
 * <p>
 * By encapsulating all the logic necessary for interrogating and
 * updating a particular View into a single helper class, we remove
 * the need for the Command classes to have any knowledge about the
 * implementation of the View.  The ViewHelper class decouples our
 * presentation from the control of the application.
 * </p>
 *
 * <p>
 * A ViewHelper belongs to a particular View in the application; when
 * a ViewHelper is created, its id is used to register
 * against a particular View component (such as a particular
 * tab in a TabNavigator, or a particular screen in a ViewStack).  The
 * developer then uses the ViewLocator to locate the particular
 * ViewHelper for interrogation or update of a particular View.
 * </p>
 * @see ViewLocator
 */
class org.nevis.cairngorm.view.ViewHelper implements mx.core.MXMLObject
{
   /**
    * Creates a ViewHelper that is used to manipulate
    * a particular viewComponent.
    *
    * @description The concrete ViewHelper class will advertise get and
    * set methods, that are responsible for updating components on the
    * view referred to by viewComponent.  Additionally, a developer may
    * choose to use a ViewHelper to provide methods to switch the
    * rendering of a given View (in a commerce application for instance,
    * a method may be exposed to toggle between a TileList or DataGrid
    * view of catalogue items).
    *
    * <p>
    * On initialization , the ViewHelper is
    * registered with the ViewLocator, to allow it to be retrieved at a
    * future point in time. If the view is unloaded (for example, the
    * view is a popup that is closed), then it is unregistered from
    * the ViewLocator, allowing it to be re-registered again if the
    * need arises.
    * </p>
    *
    * <p>
    * The ViewHelper will typically be retrieved within a Command class;
    * in the execute() method to interrogate the view, and in the
    * result handler in a Command class, to update the view following
    * successful execution of a command.
    *
    */
   public function ViewHelper()
   {
   }

//------------------------------------------------------------------------------

   /**
    * On initialization, the view is registered with the <code>ViewLocator</code>
    * with the <code>ViewLocator</code>, using its <code>id</code>.
    * On unload of the view, the view is unregistered from the <code>ViewLocator</code>.
    * @description  The <code>initialized</code> method is called by the Flex component
    * framework after a component has been initialized, so long as the
    * component implements <code>mx.core.MXMLObject</code>.
    * @see ViewLocator
    */
   public function initialized( view : Object, id : String ) : Void
   {
      this.view = view;
      this.id = id;
      ViewLocator.getInstance().register( id, this );

      view.addEventListener( "unload", Delegate.create( this, unregisterView ) );
   }

//-----------------------------------------------------------------------------

   /**
    * Unregisters the view from the ViewLocator.
    * @see ViewLocator
    */
   private function unregisterView() : Void
   {
      ViewLocator.getInstance().unregister( id );
   }

//-----------------------------------------------------------------------------

   private var view : Object;
   private var id : String;

}

