/*

Copyright 2004 iteration::two Ltd

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

@ignore
*/

import org.nevis.cairngorm.view.ViewHelper;

/**
 * The ViewLocator is a singleton class, that is used to retreive ViewHelper
 * classes that can manipulate (get/set/switch) the user interface of a
 * Cairngorm RIA.
 *
 * @description
 * Command Classes will typically require the assistance of ViewHelpers to get/set the user interface
 * associated with the work they are designed to perform.  ViewHelper classes
 * contain methods that allow them to interrogate (get) individual elements
 * of the view, such as the values of form fields, and to update the user
 * interface, such as setting the data provider on DataGrids or List
 * components.  Additionally, a Command may choose to toggle the user
 * interface from one view to another - for instance, to take the user to
 * a checkout screen.
 *
 * <p>
 * The ViewLocator class is used to allow Commands to instantly retrieve
 * the appropriate ViewHelper.  A command need only know the canonical
 * name of a ViewHelper and the ViewLocator will return an instance of
 * the appropriate ViewHelper class.  In this way, Command classes can
 * manipulate the View irrespective of its implementation.
 * </p>
 *
 * @see org.nevis.cairngorm.view.ViewHelper
 * @see org.nevis.cairngorm.commands.Command
 */
class org.nevis.cairngorm.view.ViewLocator
{

//------------------------------------------------------------------------------

   /**
    * Singleton access to the ViewLocator is assured through the static getInstance()
    * method, which is used to retrieve the only ViewLocator instance in a Cairngorm
    * application.
    *
    * @description
    * Wherever there is a need to retreive the ViewLocator instance, it is achieved
    * using the following code:
    *
    * <p>
    * <code>
    * var viewLocator:ViewLocator = ViewLocator.getInstance();
    * </code>
    * </p>
    */
   public static function getInstance() : ViewLocator
   {
      if( viewLocator == undefined )
         viewLocator = new ViewLocator();

      return viewLocator;
   }

//------------------------------------------------------------------------------

   /**
    * Private constructor ensures that ViewLocator can only be created
    * through the static singleton getInstance() method.  ViewLocator
    * maintains an array of ViewHelpers, keyed on viewName with a
    * particular view as the value stored in the array.
    */
   private function ViewLocator()
   {
      viewHelpers = new Array();
   }

//------------------------------------------------------------------------------

   /**
    * Registers a viewHelper under a canonical viewName
    *
    * @description In order that the application developer need not know
    * the implementation of the view, a ViewHelper capable of manipulating
    * a given view is registered under a simple canonical name.
    * <p>
    * For example, a LoginViewHelper may allow the manipulation of a
    * Login window, that may start life as a PopUpWindow, but later be
    * changed to a screen in a ViewStack.  By registering the LoginViewHelper
    * with the viewName "login", then any code that fetches the ViewHelper
    * by it's name "login", and then calls methods on the ViewHelper, is
    * completely insulated from any changes in the implementation of the
    * view, and the implementation of the ViewHelper.
    * </p>
    * <p>
    * If a view is already registered with the canonical name, and Error
    * is thrown.
    * </p>
    * @param viewName A simple canonical name for the view that the ViewHelper
    * will manipulate, eg "login"
    * @param viewHelper An instance of a ViewHelper
    */
   public function register( viewName:String, viewHelper:ViewHelper ) : Void
   {
      if ( registrationExistsFor( viewName ) )
         throw new Error( "View names must be unique. A view has already been registered with " + viewName );

      viewHelpers[ viewName ] = viewHelper;
   }

//------------------------------------------------------------------------------

   /**
    * Unregisters a viewHelper using its canonical name
    *
    * @param viewName The canonical name for the view to be removed
    */
   public function unregister( viewName:String ) : Void
   {
      if ( !registrationExistsFor( viewName ) )
         throw new Error( "No view has been registered with " + viewName );

      viewHelpers[ viewName ] = null;
   }

//------------------------------------------------------------------------------

   /**
    * Retrieves the ViewHelper instance that has previously been registered
    * with viewName
    *
    * @param viewName The name of the view for which we wish to retrieve a
    * ViewHelper, eg "login"
    * @returns The ViewHelper instance that is required to manipulate the
    * view registered with viewName
    */
   public function getViewHelper( viewName:String ) : ViewHelper
   {
      if ( !registrationExistsFor( viewName ) )
         throw new Error( "No view has been registered with " + viewName );

      return viewHelpers[ viewName ];
   }

//------------------------------------------------------------------------------

   /**
    * Returns whether a view has been registered with a canonical name
    *
    * @description If two views are registered with the same canonical name,
    * the second entry will overwrite the first. This method can be used to check
    * whether a view has already been registered with a canonical name.
    *
    * @param The canonical name for the view that the ViewHelper will check, eg
    * "login"
    * @return A Boolean that indicates if a view is already registered with that
    * view name
    */
   public function registrationExistsFor( viewName:String ) : Boolean
   {
      return viewHelpers[ viewName ] != undefined;
   }

//------------------------------------------------------------------------------

   private static var viewLocator : ViewLocator;
   private var viewHelpers : Object;


}